# coding=utf-8
import math
import os
import re
import time
import json
import shutil
import zipfile
from uuid import uuid4
from urllib.parse import urlparse, urlencode
from .source import routes, apis
from .page.utils import downloader, danmaku2ass
from bs4 import BeautifulSoup as Bs
from alive_progress import alive_bar
from jinja2 import PackageLoader, Environment
from acfun.libs.you_get.extractors.acfun import download as you_get_download

__author__ = 'dolacmeo'


def unix2datestr(t: (int, float), f: str = "%Y-%m-%d %H:%M:%S"):
    return time.strftime(f, time.localtime(t))


class AcSaver:
    templates = Environment(loader=PackageLoader('acfun', 'templates'))
    templates.filters['unix2datestr'] = unix2datestr
    templates.filters['math_ceil'] = math.ceil
    folder_names = ['article', 'video', 'bangumi', 'live', 'moment']
    emot_alias = ['default', 'ac', 'ac2', 'ac3', 'dog', 'tsj', 'brd', 'ais', 'td', 'zuohe', 'blizzard']
    obj2folder = {
        "AcArticle": "article",
        "AcMoment": "moment",
        "AcVideo": "video",
        "AcBangumi": "bangumi",
        "AcLive": "live",
        "AcUp": "member"
    }
    ubb_tag_basic = {
        r"\r\n": "<br>",
        "[b]": "<b>", "[/b]": "</b>",
        "[i]": "<i>", "[/i]": "</i>",
        "[u]": "<u>", "[/u]": "</u>",
        "[s]": "<s>", "[/s]": "</s>",
        "[/color]": r"</color>"
    }
    ubb_tag_rex = {
        "color": r"(\[color=(#[a-f0-9]{6})\])",
        "size": r"(\[size=(\d+px)\]([^\[]+)\[/size\])",
        "emot": r"(\[emot=acfun,(\S+?)\/])",
        "emot_old": r"(\[emot=([a-z0-9]+),(\S+?)\/])",
        "image": r"(\[img=\\u56fe\\u7247\]([^\[]*)\[\/img\])",
        "at": r"(\[at uid=(\d+)\](@[^\[]+)\[/at\])",
        "at_old": r"(\[at\]([^\[]+)\[/at\])",
        "resource": r"(\[resource id=(\d+) type=([1-5]) icon=[^\]]*\]([^\[]*)\[\/resource\])",
        "jump": r"(\[time duration=(\d+)\]([^\[]+)\[/time\])",
    }
    ubb_resource_url = {
        "1": routes['bangumi'],
        "2": routes['video'],
        "3": routes['article'],
        "4": routes['album'],
        "5": routes['up']
    }
    ubb_resource_icon = {
        "1": r'<img class=\"ubb-icon\" src=\"../../assets/img/icon_comment_video_pc.png\">',
        "2": r'<img class=\"ubb-icon\" src=\"../../assets/img/icon_comment_pc_vid_18_3.png\">',
        "3": r'<img class=\"ubb-icon\" src=\"../../assets/img/icon_popup_article_pc.png\">',
        "4": r'<img class=\"ubb-icon\" src=\"../../assets/img/icon_comment_heji_pc.png\">',
        "5": r'<img class=\"ubb-icon\" src=\"../../assets/img/icon_comment_human_pc.png\">',
    }

    def __init__(self, acer, ac_obj, dest_path=None):
        self.acer = acer
        self.ac_obj = ac_obj
        self.dest_path = dest_path or os.path.join(os.getcwd(), 'download')
        self.folder_name = "else" if ac_obj is None else self.obj2folder.get(self.ac_obj.__class__.__name__, "else")
        if not os.path.isdir(self.dest_path):
            os.makedirs(self.dest_path, exist_ok=True)
        self.cdns = self.acer.client.post(apis['cdn_domain'], headers={
            "referer": routes['index']}).json().get('domain')
        self._check_folders()
        # self._check_assets()

    def _check_folders(self):
        checks = list()
        for x in self.folder_names:
            x_path = os.path.join(self.dest_path, x)
            os.makedirs(x_path, exist_ok=True)
            checks.append(os.path.isdir(x_path))
        os.makedirs(os.path.join(self.dest_path, 'member'), exist_ok=True)
        index_html = self.templates.get_template('index.html').render()
        index_html_path = os.path.join(self.dest_path, 'index.html')
        if os.path.isfile(index_html_path) is False:
            self._check_assets()
        with open(index_html_path, 'wb') as i:
            i.write(index_html.encode())
        return all(checks)

    def _check_assets(self):
        if os.path.isdir(os.path.join(self.dest_path, 'assets')) is False:
            return False
        checks = list()
        checks.append(os.path.isfile(os.path.join(self.dest_path, 'assets', 'favicon.ico')))
        for n in ['css', 'js', 'img', 'font', 'emot']:
            checks.append(os.path.isdir(os.path.join(self.dest_path, 'assets', n)))
        return all(checks)

    def download_assets_from_github(self):
        assert_zip_path = os.path.join(self.dest_path, 'assets.zip')
        assert_zip_saved = os.path.isfile(assert_zip_path)
        if assert_zip_saved is False:
            assets_url = r"https://archive.fastgit.org/dolaCmeo/acfunSDK/archive/refs/heads/assets.zip"
            assert_download = self._download(assets_url, display=True)
            if assert_download is None:
                return False
        zip_file = zipfile.ZipFile(assert_zip_path)
        zip_file.extractall(self.dest_path)
        assert_zip_folder = os.path.join(self.dest_path, 'acfunSDK-assets', 'assets')
        assert_folder = os.path.join(self.dest_path, 'assets')
        shutil.move(assert_zip_folder, assert_folder)
        shutil.rmtree(os.path.join(self.dest_path, 'acfunSDK-assets'), ignore_errors=True)
        assert_folder_ok = os.path.isdir(assert_folder)
        if assert_folder_ok is True:
            emot_map_path = os.path.join(self.dest_path, 'assets', 'emot', 'emotion_map.json')
            if os.path.isfile(emot_map_path) is False:
                self.save_emot()
        return assert_folder_ok

    def folder_list_update(self):
        jsFiles = []
        for fn in self.folder_names:
            fpath = os.path.join(self.dest_path, fn)
            f_all = os.listdir(fpath)
            f_all = [i for i in f_all if os.path.isdir(os.path.join(fpath, i))]
            f_all_string = json.dumps(f_all, separators=(',', ':'))
            nums_js = f"let {fn}Nums={f_all_string};"
            js_path = os.path.join(fpath, 'nums.js')
            with open(os.path.join(fpath, 'nums.js'), 'wb') as js:
                js.write(nums_js.encode())
            jsFiles.append(os.path.isfile(js_path))
        return all(jsFiles)

    def record_last(self):
        ac_num = f"ac{self.ac_obj.ac_num}"
        if "_" in ac_num:
            ac_num = ac_num.split('_')[0]

        last_data = []
        last_path = os.path.join(self.dest_path, self.folder_name, 'leatest.js')
        if os.path.isfile(last_path):
            sn = len(f"{self.folder_name}Last=")
            last_text = open(last_path, 'r').read()
            last_data = json.loads(last_text.strip()[sn:-1])
        if ac_num not in last_data:
            last_data.append(ac_num)
        last_string = json.dumps(last_data, separators=(',', ':'))
        last_js = f"{self.folder_name}Last={last_string};"
        with open(last_path, 'wb') as js:
            js.write(last_js.encode())

        recent_data = []
        recent_path = os.path.join(self.dest_path, 'assets', 'data', 'recent.js')
        if os.path.isfile(recent_path):
            recent_text = open(recent_path, 'r').read()
            recent_data = json.loads(recent_text.strip()[12:-1])
        if [self.folder_name, ac_num] not in recent_data:
            recent_data.append([self.folder_name, ac_num])
        recent_string = json.dumps(recent_data, separators=(',', ':'))
        recent_js = f"AcCacheList={recent_string};"
        with open(recent_path, 'wb') as js:
            js.write(recent_js.encode())

        return all([os.path.isfile(last_path), os.path.isfile(recent_path)])

    def _setup_folder(self):
        folder_path = os.path.join(self.dest_path, self.folder_name, f"ac{self.ac_obj.ac_num}")
        os.makedirs(os.path.join(folder_path, 'data'), exist_ok=True)
        if self.folder_name in ["article", "moment", "video", "bangumi"]:
            os.makedirs(os.path.join(folder_path, 'img'), exist_ok=True)
        else:
            os.makedirs(folder_path, exist_ok=True)
        return folder_path

    def _renew_folder(self, abs_path: str):
        os.makedirs(os.path.dirname(abs_path), exist_ok=True)
        if os.path.isdir(abs_path):
            shutil.rmtree(abs_path, True)
        os.makedirs(abs_path)

    def _download(self, src_url: str, fname: [str, None] = None, ex_dir: [list, None] = None, display: bool = False):
        save_path = self.dest_path if ex_dir is None else os.path.join(self.dest_path, *ex_dir)
        return downloader(self.acer.client, src_url, fname, save_path, display)

    def save_emot(self):
        emot_req = self.acer.client.post(apis['emot'])
        emot_data = emot_req.json()
        assert emot_data['result'] == 0
        emot_ex_dir = ['assets', 'emot']
        emot_dir = os.path.join(self.dest_path, *emot_ex_dir)
        os.makedirs(emot_dir, exist_ok=True)
        emot_json_saved = self._json_saver(emot_data, "emotion.json", emot_dir)
        for pack in emot_data['emotionPackageList']:
            if len(pack['downloadUrl']):
                fname = urlparse(pack['downloadUrl']).path.split('/')[-1]
                zip_path = os.path.join(emot_dir, f"{fname}.zip")
                if os.path.isfile(zip_path) is False:
                    self._download(pack['downloadUrl'], f"{fname}.zip", emot_ex_dir)
                if os.path.isfile(zip_path):
                    zip_file = zipfile.ZipFile(zip_path)
                    zip_file.extractall(os.path.dirname(zip_path))
            for em in pack['emotions']:
                for x, y in {'small': 'smallImageInfo', 'big': 'bigImageInfo'}.items():
                    em_src = em[y]['thumbnailImageCdnUrl']
                    em_ext = urlparse(em_src).path.split('/')[-1].split('.')[-1]
                    em_path = os.path.join(emot_dir, x, f"{em['id']}.{em_ext}")
                    if not os.path.isfile(em_path):
                        self._download(em_src, f"{em['id']}.{em_ext}", ['assets', 'emot', x])
        local_emot_list = os.listdir(os.path.join(emot_dir, 'big'))
        for n in range(1, 56):
            if f"{n}.gif" not in local_emot_list:
                src = f"https://cdnfile.aixifan.com/static/umeditor/emotion/images/ac/{n}.gif"
                self._download(src, f"{n}.gif", ['assets', 'emot', 'big'])
        emot_map_path = os.path.join(emot_dir, "emotion_map.json")
        emot_map = {"saved": {}, "lost": []}
        if os.path.isfile(emot_map_path) is True:
            emot_map = json.load(open(emot_map_path, 'rb'))
        else:
            for emot_name in os.listdir(emot_dir):
                this_path = os.path.join(emot_dir, emot_name)
                if os.path.isdir(this_path) is False or emot_name in ['small']:
                    continue
                files = os.listdir(this_path)
                emot_map['saved'].update({
                    f"acfun,{x.split('.')[0]}" if emot_name == 'big' else f"{emot_name},{x.split('.')[0]}":
                        f"../../assets/emot/{emot_name}/{x}"
                    for x in files
                })
        for alias in self.emot_alias:
            alias_emot_path = os.path.join(emot_dir, alias)
            os.makedirs(alias_emot_path, exist_ok=True)
            alias_emot_list = os.listdir(alias_emot_path)
            for n in range(1, 60):
                if n < 10:
                    map_name0 = f"{alias},{n}"
                    if f"{n}.gif" in alias_emot_list:
                        emot_map['saved'].update({f"{map_name0}":  f"../../assets/emot/{alias}/{n}.gif"})
                        continue
                    elif map_name0 in emot_map['lost']:
                        continue
                    src = f"https://cdnfile.aixifan.com/static/umeditor/emotion/images/{alias}/{n}.gif"
                    name0_save = self._download(src, f"{n}.gif", ['assets', 'emot', alias])
                    if name0_save is None:
                        emot_map['lost'].append(map_name0)
                    elif os.path.isfile(name0_save):
                        emot_map['saved'].update({f"{map_name0}": f"../../assets/emot/{alias}/{n}.gif"})
                    else:
                        print("EMOT UNKNOWN:", map_name0)
                map_name1 = f"{alias},{n:0>2}"
                if f"{n:0>2}.gif" in alias_emot_list:
                    emot_map['saved'].update({f"{map_name1}":  f"../../assets/emot/{alias}/{n:0>2}.gif"})
                    continue
                elif map_name1 in emot_map['lost']:
                    continue
                src = f"https://cdnfile.aixifan.com/static/umeditor/emotion/images/{alias}/{n:0>2}.gif"
                name1_save = self._download(src, f"{n:0>2}.gif", ['assets', 'emot', alias])
                if name1_save is None:
                    emot_map['lost'].append(map_name1)
                elif os.path.isfile(name1_save):
                    emot_map['saved'].update({f"{map_name1}": f"../../assets/emot/{alias}/{n:0>2}.gif"})
                else:
                    print("EMOT UNKNOWN:", map_name1)
        emot_map_saved = self._json_saver(emot_map, "emotion_map.json", emot_dir)
        return all([emot_json_saved, emot_map_saved])

    def _save_data(self, data: [dict, list, str], fname: str, ex_dir: [list, None] = None):
        if isinstance(data, (dict, list)):
            data = json.dumps(data, separators=(',', ':'))
        elif isinstance(data, str):
            pass
        else:
            return False
        save_path = self.dest_path if ex_dir is None else os.path.join(self.dest_path, *ex_dir)
        file_path = os.path.join(save_path, fname)
        if not file_path.endswith('.json'):
            file_path += '.json'
        with open(file_path, 'w') as save_file:
            save_file.write(data)
        return os.path.exists(file_path)

    def _save_images(self, src_url: str, fname: [str, None] = None, ex_dir: [list, None] = None):
        src_uu = urlparse(src_url)
        if fname is None:
            fname = src_uu.path.split('/')[-1]
            if src_url.startswith("https://www.acfun.cn/imageProxy?"):
                fname = uuid4().hex.upper()
            # elif src_uu.netloc in self.cdns:
            #     fname = src_uu.path.split('/')[-1]
        return self._download(src_url, fname, ex_dir)

    def _save_member(self, ids: list, force: bool = False):
        if len(ids) == 0:
            return []
        done = list()
        member_dir = os.path.join(self.dest_path, 'member')
        saved = os.listdir(member_dir)
        ids = sorted(list(set(ids)))
        ids_with_ext = [f"{i}.json" for i in ids]
        ids = list(set(ids_with_ext).difference([x for x in saved if x.endswith('.json')]))
        ids = [y.split('.')[0] for y in ids]
        with alive_bar(len(ids), length=30, disable=len(ids) < 5, title="save members",
                       force_tty=True, stats=False) as progress:
            for uid in ids:
                if all([f"{uid}.json" in saved, f"{uid}.js" in saved, f"{uid}_avatar" in saved]) is True \
                        and force is False:
                    progress()
                    done.append(uid)
                    continue
                user_req = self.acer.client.get(apis['userInfo'], params=dict(userId=uid))
                user_data = user_req.json()
                profile = user_data.get('profile')
                user_json = os.path.join(member_dir, f"{uid}.json")
                user_js = os.path.join(member_dir, f"{uid}.js")
                user_avatar = os.path.join(member_dir, f"{uid}_avatar")
                if all([os.path.isfile(user_json), os.path.isfile(user_js), os.path.isfile(user_avatar)]) is True \
                        and force is False:
                    progress()
                    done.append(uid)
                    continue
                with open(user_json, 'w') as uid_file:
                    json.dump(profile, uid_file, separators=(',', ':'))
                user_json_saved = os.path.isfile(user_json)
                user_json_string = open(os.path.join(user_json), 'rb').read().decode()
                with open(user_js, 'wb') as js_file:
                    user_js_string = f"let user_{uid}={user_json_string};"
                    js_file.write(user_js_string.encode())
                user_js_saved = os.path.isfile(user_js)
                avatar = urlparse(profile['headUrl'])
                if avatar.path.endswith('defaultAvatar.jpg'):
                    avatar_saved = True
                else:
                    avatar_path = self._save_images(f"{avatar.scheme}://{avatar.netloc}{avatar.path}", str(uid), ['member'])
                    avatar_saved = False if avatar_path is None else os.path.isfile(avatar_path)
                    if avatar_saved:
                        shutil.move(avatar_path, user_avatar)
                if all([user_json_saved, user_js_saved, avatar_saved]):
                    done.append(uid)
                progress()
                time.sleep(0.1)
        return done

    def get_user(self, uid: str):
        self._save_member([uid], True)
        user_json_path = os.path.join(self.dest_path, 'member', f"{uid}.json")
        user_data = json.load(open(user_json_path, 'rb'))
        return user_data

    def _json_saver(self, data: dict, filename: str, dest_path=None):
        folder_path = self._setup_folder()
        json_path = os.path.join(folder_path, 'data', filename)
        if dest_path is not None:
            json_path = os.path.join(dest_path, filename)
        json_string = json.dumps(data, separators=(',', ':'))
        with open(json_path, 'wb') as json_file:
            json_file.write(json_string.encode())
        result = os.path.isfile(json_path)
        print("SAVED:", result, json_path)
        return result

    def _save_comment(self, update: bool = False):
        folder_path = self._setup_folder()
        local_comment_data, local_comment_floors = None, []
        comment_json_path = os.path.join(folder_path, 'data', f"ac{self.ac_obj.ac_num}.comment.json")
        comment_json_path_saved = os.path.isfile(comment_json_path)
        if comment_json_path_saved:
            print("loading comment data.json")
            local_comment_data = json.load(open(comment_json_path, 'rb'))
            local_comment_floors = [x['floor'] for x in local_comment_data['rootComments']]
        if update is True or comment_json_path_saved is False:
            comment_obj = self.ac_obj.comment()
            comment_obj.get_all_comment()
            if local_comment_data is not None:
                comment_data = local_comment_data
                comment_data['hotComments'] = comment_obj.hot_comments
                comment_data['rootComments'].extend(
                    [c for c in comment_obj.root_comments if c['floor'] not in local_comment_floors])
                comment_data['subCommentsMap'].update(comment_obj.sub_comments)
                comment_data['save_unix'] = time.time()
            else:
                comment_data = {
                    "hotComments": comment_obj.hot_comments,
                    "rootComments": comment_obj.root_comments,
                    "subCommentsMap": comment_obj.sub_comments,
                    "save_unix": time.time()
                }
            uids = list()
            for c in comment_data['rootComments']:
                if c['userId'] not in uids:
                    uids.append(c['userId'])
            for _, i in comment_data['subCommentsMap'].items():
                for j in i['subComments']:
                    if j['userId'] not in uids:
                        uids.append(j['userId'])
            self._save_member(uids)
            comment_saved = self._json_saver(comment_data, f"ac{self.ac_obj.ac_num}.comment.json")
            print("SAVED:", comment_saved, comment_json_path)
        else:
            comment_saved = True
        comment_js_saved = self._tans_comment_uub2html()
        return all([comment_saved, comment_js_saved])

    def _tans_comment_uub2html(self):
        print("loading comment data json")
        folder_path = self._setup_folder()
        comment_json_path = os.path.join(folder_path, 'data', f"ac{self.ac_obj.ac_num}.comment.json")
        comment_json_temp = os.path.join(folder_path, 'data', f"ac{self.ac_obj.ac_num}.comment.temp")
        temp_ok = os.path.isfile(comment_json_temp)
        if temp_ok:
            print('loading temp')
            comment_json_string = open(comment_json_temp, 'r').read()
        else:
            comment_json_string = open(comment_json_path, 'r').read()
            print('process comment ubb tags')
            # 基础替换：换行,加粗,斜体,下划线,删除线,颜色结尾
            for ubb, tag in self.ubb_tag_basic.items():
                comment_json_string = comment_json_string.replace(ubb, tag)
            # 正则替换：颜色,表情,图片
            emot_map_path = os.path.join(self.dest_path, 'assets', 'emot', 'emotion_map.json')
            if os.path.isfile(emot_map_path) is False:
                self.save_emot()
            emot_map = json.load(open(emot_map_path, 'r'))
            for n, rex_rule in self.ubb_tag_rex.items():
                for tag in re.compile(rex_rule).findall(comment_json_string):
                    if n == 'color':
                        comment_json_string = comment_json_string.replace(
                            tag[0], f'<font color=\\"{tag[1]}\\">')
                    elif n == 'size':
                        comment_json_string = comment_json_string.replace(
                            tag[0], f'<span style=\\"font-size:{tag[1]}\\">{tag[2]}</span>')
                    elif n == 'emot':
                        if tag[1] in emot_map:
                            comment_json_string = comment_json_string.replace(
                                tag[0], f'<img class=\\"ubb-emotion\\" src=\\"{emot_map[tag[1]]}\\">')
                    elif n == 'emot_old':
                        alias = ",".join(tag[1:])
                        if alias in emot_map['saved']:
                            comment_json_string = comment_json_string.replace(
                                tag[0], f'<img class=\\"ubb-emotion\\" src=\\"{emot_map["saved"][alias]}\\">')
                        elif alias in emot_map['lost']:
                            pass
                        else:
                            print("??? emot:", tag)
                    elif n == 'image':
                        img_path = self._save_images(tag[1], ex_dir=[self.folder_name, f"ac{self.ac_obj.ac_num}", 'img'])
                        if img_path is None:
                            print(f"IMG ERROR: {tag[1]}")
                            comment_json_string = comment_json_string.replace(
                                tag[0], f'<img class=\\"lazy\\" data-src=\\"../../assets/img/404img.png\\" alt=\\"{tag[1]}\\">')
                        else:
                            img_name = os.path.basename(img_path)
                            comment_json_string = comment_json_string.replace(
                                tag[0], f'<img class=\\"lazy\\" data-src=\\"img/{img_name}\\" alt=\\"{tag[1]}\\">')
                    elif n == 'at':
                        comment_json_string = comment_json_string.replace(
                            tag[0], f'<a class=\\"ubb-name\\" target=\\"_blank\\" href=\\"https://www.acfun.cn/u/{tag[1]}\\">{tag[2]}</a>')
                    elif n == 'at_old':
                        comment_json_string = comment_json_string.replace(
                            tag[0], f'<a class=\\"ubb-name\\">@{tag[1]}</a>')
                    elif n == 'jump':
                        comment_json_string = comment_json_string.replace(
                            tag[0], f'<a class=\\"quickJump\\" onclick=\\"quickJump({tag[1]})\\">{tag[2]}</a>')
                    elif n == 'resource':
                        resource_a = '<a class=\\"ubb-ac\\" data-aid=\\"{ac_num}\\" href=\\"{href}\\" target=\\"_blank\\">{title}</a>'
                        comment_json_string = comment_json_string.replace(
                            tag[0], self.ubb_resource_icon[tag[2]]+resource_a.format(
                                ac_num=tag[1],
                                href=self.ubb_resource_url[tag[2]]+tag[1],
                                title=tag[3]
                            ))
            # for tag in re.compile(r"(\[([^\]]+)\])").findall(comment_json_string):
            #     print(tag)
            print('SAVE TEMP')
            with open(comment_json_temp, 'w') as t:
                t.write(comment_json_string)
        print('split comment data')
        comment_data = json.loads(comment_json_string)
        total_comment = len(comment_data['rootComments'])
        # 评论分块存储，每块100条；跟楼按每页划分。
        # 区块正向划分，预留已删除位置；区块顺序列表倒置；热评在最后。
        total_block = math.ceil(total_comment / 100)
        blocks = {}
        count = 0
        for X in comment_data['rootComments']:
            count += 1
            z = str(math.floor(X['floor'] / 100))
            if z not in blocks:
                blocks[z] = {
                    "hotComments": [],
                    "rootComments": [],
                    "subCommentsMap": {},
                    "save_unix": time.time(),
                    "totalComment": total_comment
                }
            blocks[z]['rootComments'].append(X)
            cid = str(X['commentId'])
            if cid in comment_data['subCommentsMap']:
                blocks[z]['subCommentsMap'][cid] = comment_data['subCommentsMap'][cid]
        totals = 0
        for v in blocks.values():
            totals += len(v["rootComments"])
        blocks = [j for i, j in sorted(blocks.items(), reverse=True)]
        blocks[0]["hotComments"] = comment_data["hotComments"]
        for Y in blocks[0]["hotComments"]:
            cid = str(Y['commentId'])
            if cid not in blocks[0]['subCommentsMap'] and cid in comment_data['subCommentsMap']:
                blocks[0]['subCommentsMap'][cid] = comment_data['subCommentsMap'][cid]
        for i in range(len(blocks)):
            B = blocks[i]
            B.update({'page': i + 1, 'total': len(blocks)})
            B['rootComments'] = sorted(B['rootComments'], key=lambda x: x['floor'], reverse=True)
            comment_block_js_path = os.path.join(folder_path, 'data', f"ac{self.ac_obj.ac_num}.comment.{i+1}.js")
            comment_block_js_string = json.dumps(B, separators=(',', ':'))
            with open(comment_block_js_path, 'wb') as js_file:
                comment_js = f"commentData[{i+1}]={comment_block_js_string};"
                js_file.write(comment_js.encode())
            print("SAVED:", os.path.isfile(comment_block_js_path), comment_block_js_path)
        if temp_ok:
            os.remove(comment_json_temp)
        return os.path.isfile(comment_json_path)

    def _save_danmaku(self, num: int = 1):
        assert num <= len(self.ac_obj.video_list)
        v_num = f"{self.ac_obj.ac_num}_{num}" if num > 1 else f"{self.ac_obj.ac_num}"
        folder_path = self._setup_folder()
        self.ac_obj.set_video(num)
        danmaku_obj = self.ac_obj.danmaku()
        danmaku_saved = self._json_saver(danmaku_obj.danmaku_data, f"ac{v_num}.danmaku.json")
        danmaku_json_string = open(os.path.join(folder_path, 'data', f"ac{v_num}.danmaku.json"), 'rb').read().decode()
        danmaku_js_path = os.path.join(folder_path, 'data', f"ac{v_num}.danmaku.js")
        with open(danmaku_js_path, 'wb') as js_file:
            danmaku_js = f"let ac{v_num}_danmaku={danmaku_json_string};"
            js_file.write(danmaku_js.encode())
        danmaku_js_saved = os.path.isfile(danmaku_js_path)
        print("SAVED:", danmaku_js_saved, danmaku_js_path)
        danmaku_ass_path = self._danmaku2ass(num)
        if danmaku_ass_path is None:
            danmaku_ass_saved, danmaku_ass_js_saved = True, True
        else:
            danmaku_ass_saved = os.path.isfile(danmaku_ass_path)
            danmaku_ass_js_path = os.path.join(folder_path, 'data', f"ac{v_num}.ass.js")
            danmaku_ass_js_saved = os.path.isfile(danmaku_ass_js_path)
            print("SAVED:", danmaku_ass_js_saved, danmaku_ass_js_path)
        return all([danmaku_saved, danmaku_js_saved, danmaku_ass_saved, danmaku_ass_js_saved])

    def _danmaku2ass(self, num: int = 1):
        assert num <= len(self.ac_obj.video_list)
        v_num = f"{self.ac_obj.ac_num}_{num}" if num > 1 else f"{self.ac_obj.ac_num}"
        folder_path = self._setup_folder()
        self.ac_obj.set_video(num)
        return danmaku2ass(self.acer.client, folder_path, f"ac{v_num}")

    def _save_qrcode(self, url: [str, None] = None, filename: str = "share_qrcode.png"):
        size = 200
        param = {
            "content": url or self.ac_obj.share_url,
            "contentType": "URL",
            "toShortUrl": False,
            "width": size,
            "height": size
        }
        qr_url = f"{apis['qrcode']}?{urlencode(param)}"
        return self._save_images(qr_url, filename, ex_dir=[self.folder_name, f"ac{self.ac_obj.ac_num}", 'data'])


class ArticleSaver(AcSaver):

    def __init__(self, acer, ac_obj, dest_path=None):
        super().__init__(acer, ac_obj, dest_path)
        self.folder_path = self._setup_folder()
        self.v_num = f"{self.ac_obj.ac_num}"

    def _save_base_data(self):
        content_raw_saved = self._json_saver(self.ac_obj.article_data, f"ac{self.v_num}.json")
        content_json_string = open(os.path.join(self.folder_path, 'data', f"ac{self.v_num}.json"), 'rb').read().decode()
        content_js_path = os.path.join(self.folder_path, 'data', f"ac{self.v_num}.js")
        with open(content_js_path, 'wb') as js_file:
            content_js = f"articles['ac{self.v_num}']={content_json_string};"
            js_file.write(content_js.encode())
        content_js_saved = os.path.isfile(content_js_path)
        return all([content_raw_saved, content_js_saved])

    def _save_page(self):
        cover_path = self._save_images(self.ac_obj.article_data['coverUrl'], 'cover',
                                       [self.folder_name, f"ac{self.ac_obj.ac_num}"])
        cover_saved = os.path.isfile(cover_path)
        shutil.copy(cover_path, os.path.join(self.dest_path, self.folder_path, f"cover._"))
        share_qrcode_path = self._save_qrcode()
        share_qrcode_saved = os.path.isfile(share_qrcode_path)
        up_data = self.get_user(self.ac_obj.article_data['user']['id'])
        article_template = self.templates.get_template('article.html')
        article_html = article_template.render(
            up_reg_date=unix2datestr(up_data['registerTime']),
            v_num=self.v_num, up_data=up_data, RAW=self.ac_obj.article_data)
        html_obj = Bs(article_html, 'lxml')
        html_img_path = [self.folder_name, f"ac{self.ac_obj.ac_num}", 'img']
        self._renew_folder(os.path.join(self.folder_path, 'img'))
        for img in html_obj.select('.article-content img'):
            if img.attrs['src'].startswith('http') or img.attrs['src'].startswith('//'):
                saved_path = self._save_images(img.attrs['src'], ex_dir=html_img_path)
                img.attrs['alt'] = img.attrs['src']
                img.attrs['class'] = "lazy"
                img.attrs['src'] = "../../assets/img/logo-gray.png"
                img.attrs['data-src'] = f"./img/{os.path.basename(saved_path)}"
        html_path = os.path.join(self.dest_path, self.folder_path, f'ac{self.v_num}.html')
        with open(html_path, 'wb') as html_file:
            html_file.write(html_obj.prettify().encode())
        content_html_saved = os.path.isfile(html_path)
        return all([cover_saved, share_qrcode_saved, content_html_saved])

    def save_all(self):
        assert os.path.isfile(os.path.join(self.dest_path, 'assets', 'emot', 'emotion_map.json')) is True
        # 保存文章数据
        data_saved = self._save_base_data()
        # 保存文章封面、二维码、页面与图片
        page_saved = self._save_page()
        # 保存作者信息
        up_saved = self._save_member([self.ac_obj.article_data['user']['id']])
        # 保存评论
        comment_saved = self._save_comment()
        done = all([
            data_saved,
            page_saved,
            comment_saved
        ])
        if done:
            self.record_last()
            self.folder_list_update()
        print(self.folder_path)
        return done

    pass


class MomentSaver(AcSaver):

    def _save_content(self):
        pass

    pass


class VideoSaver(AcSaver):

    def __init__(self, acer, ac_obj, dest_path=None):
        super().__init__(acer, ac_obj, dest_path)
        self.folder_path = self._setup_folder()

    def _save_base_data(self, num: int = 1):
        v_num = f"{self.ac_obj.ac_num}_{num}" if num > 1 else f"{self.ac_obj.ac_num}"
        video_raw_saved = self._json_saver(self.ac_obj.video_data, f"ac{v_num}.json")
        video_json_string = open(os.path.join(self.folder_path, 'data', f"ac{v_num}.json"), 'rb').read().decode()
        video_js_path = os.path.join(self.folder_path, 'data', f"ac{v_num}.js")
        with open(video_js_path, 'wb') as js_file:
            content_js = f"videos['ac{v_num}']={video_json_string};"
            js_file.write(content_js.encode())
        video_js_saved = os.path.isfile(video_js_path)
        print("SAVED:", video_js_saved, video_js_path)
        return all([video_raw_saved, video_js_saved])

    def _save_page(self, num: int = 1):
        v_num = f"{self.ac_obj.ac_num}_{num}" if num > 1 else f"{self.ac_obj.ac_num}"
        cover_path = self._save_images(
            self.ac_obj.video_data['coverUrl'], 'cover', [self.folder_name, f"ac{self.ac_obj.ac_num}"])
        cover_saved = False if cover_path is None else os.path.isfile(cover_path)
        cover__path = os.path.join(self.dest_path, self.folder_path, f"cover._")
        if cover_saved:
            shutil.copy(cover_path, cover__path)
        else:
            ac404 = os.path.join(self.dest_path, 'assets', 'img', 'default_cover.png')
            shutil.copy(ac404, os.path.join(self.dest_path, self.folder_path, f"cover._"))
            cover_saved = os.path.isfile(cover__path)
        print("SAVED:", cover_saved, cover__path)
        if num == 1:
            share_qrcode_path = self._save_qrcode()
            share_qrcode_saved = os.path.isfile(share_qrcode_path)
            mobile_qrcode_path = self._save_qrcode(self.ac_obj.mobile_url, "mobile_qrcode.png")
            mobile_qrcode_saved = os.path.isfile(mobile_qrcode_path)
        else:
            cover_saved, share_qrcode_saved, mobile_qrcode_saved = True, True, True
        up_data = self.get_user(self.ac_obj.video_data['user']['id'])
        video_template = self.templates.get_template('video.html')
        video_html = video_template.render(
            up_reg_date=unix2datestr(up_data['registerTime']), partNum=num,
            v_num=v_num, up_data=up_data, RAW=self.ac_obj.video_data)
        html_path = os.path.join(self.dest_path, self.folder_path, f"ac{v_num}.html")
        with open(html_path, 'wb') as html_file:
            html_file.write(video_html.encode(errors='ignore'))
        video_html_saved = os.path.isfile(html_path)
        print("SAVED:", video_html_saved, html_path)
        return all([cover_saved, share_qrcode_saved, mobile_qrcode_saved, video_html_saved])

    def save_video(self, num: int = 1):
        assert 0 < num <= len(self.ac_obj.video_list)
        v_num = f"{self.ac_obj.ac_num}_{num}" if num > 1 else f"{self.ac_obj.ac_num}"
        # 保存视频数据
        data_saved = self._save_base_data(num)
        # 保存视频封面、二维码、页面
        page_saved = self._save_page(num)
        # 保存作者信息
        uids = [self.ac_obj.video_data['user']['id']]
        staff = self.ac_obj.video_data.get('staffInfos', [])
        if len(staff):
            uids.extend([user['href'] for user in staff])
        up_saved = self._save_member(uids)
        # 保存视频弹幕
        danmaku_saved = self._save_danmaku(num)
        # 保存视频文件
        acfun_url = f"{routes['video']}{v_num}"
        video_path = os.path.join(self.folder_path, f"ac{v_num}.mp4")
        video_saved = os.path.isfile(video_path)
        if video_saved is False:
            you_get_download(acfun_url, output_dir=self.folder_path, merge=True)
            video_saved = os.path.isfile(video_path)
        print("SAVED:", video_saved, video_path)
        # 保存评论
        comment_saved = self._save_comment() if num == 1 else True
        done = all([
            data_saved,
            page_saved,
            danmaku_saved,
            video_saved,
            comment_saved
        ])
        if done:
            self.record_last()
            self.folder_list_update()
        print(self.folder_path)
        return done

    def save_all(self):
        result = []
        for i in range(len(self.ac_obj.video_list)):
            result.append(self.save_video(i + 1))
        return all(result)

    pass


class BangumiSaver(AcSaver):

    def _save_season(self):
        pass

    def _save_episode(self):
        pass

    pass


class LiveSaver(AcSaver):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['acfun',
 'acfun.libs',
 'acfun.libs.blackboxprotobuf',
 'acfun.libs.blackboxprotobuf.libs',
 'acfun.libs.blackboxprotobuf.libs.types',
 'acfun.libs.you_get',
 'acfun.libs.you_get.cli_wrapper',
 'acfun.libs.you_get.cli_wrapper.downloader',
 'acfun.libs.you_get.cli_wrapper.openssl',
 'acfun.libs.you_get.cli_wrapper.player',
 'acfun.libs.you_get.cli_wrapper.transcoder',
 'acfun.libs.you_get.extractors',
 'acfun.libs.you_get.processor',
 'acfun.libs.you_get.util',
 'acfun.page',
 'acfun.protos',
 'acfun.protos.Im',
 'acfun.protos.Live']

package_data = \
{'': ['*'],
 'acfun': ['src/README.md',
           'src/README.md',
           'src/generate.bat',
           'src/generate.bat',
           'templates/*',
           'templates/parts/*']}

install_requires = \
['alive-progress>=2.4,<3.0',
 'beautifulsoup4>=4.11,<5.0',
 'cssutils>=2.6,<3.0',
 'filetype>=1.1,<2.0',
 'httpx>=0.23,<0.24',
 'jinja2>=3.1,<4.0',
 'lxml>=4.9,<5.0',
 'proto-plus==1.22.1',
 'protobuf==3.20.1',
 'psutil>=5.9,<6.0',
 'pycryptodome>=3.15,<4.0',
 'rich>=12.5,<13.0',
 'websocket-client>=1.4,<2.0']

setup_kwargs = {
    'name': 'acfunsdk',
    'version': '0.8.4',
    'description': 'acfunSDK - UNOFFICEICAL',
    'long_description': '# acfunSDK - **UNOFFICEICAL**\n\n<br />\n\n<p align="center">\n<a href="https://github.com/dolaCmeo/acfunSDK">\n<img height="100" src="https://s3.dualstack.us-east-2.amazonaws.com/pythondotorg-assets/media/files/python-logo-only.svg" alt="">\n<img height="100" src="https://ali-imgs.acfun.cn/kos/nlav10360/static/common/widget/header/img/acfunlogo.11a9841251f31e1a3316.svg" alt="">\n</a>\n</p>\n\n<br />\n\nacfunSDK是 **非官方的 [AcFun弹幕视频网][acfun.cn]** Python库。\n\n几乎搜集了所有与 [AcFun弹幕视频网][acfun.cn] 相关的接口与数据。\n\nps: _如发现未知接口，或现有功能失效，请随时提交 [Issue]_\n\n- - -\n\n**Python** : 开发环境为 `Python 3.8.10` & `Python 3.9.6`\n\n`Python`本体请自行[下载安装][python]。\n\n## [从PyPI安装](https://pypi.org/project/acfunsdk/)\n\n```shell\npython -m pip install acfunsdk\n```\n\n**需要`ffmpeg`**  主要用于下载视频。\n> 建议去官网下载 https://ffmpeg.org/download.html\n>\n> 可执行文件 `ffmpeg` 需要加入到环境变量，或复制到运行根目录。\n\n- - -\n\n## 使用方法\n\n\n### 实例化获取对象\n```python\nfrom acfun import Acer\n# 实例化一个Acer\nacer = Acer(debug=True)\n# 登录用户(成功登录后会自动保存 \'<用户名>.cookies\')\n# 请注意保存，防止被盗\nacer.login(username=\'you@email.com\', password=\'balalabalala\')\n# 读取用户(读取成功登录后保存的 \'<用户名>.cookies\')\nacer.loading(username=\'13800138000\')\n# 每日签到，领香蕉🍌\nacer.signin()\n# 通过链接直接获取内容对象\n# 目前支持 9种类型：\n# 视  频: https://www.acfun.cn/v/ac4741185\ndemo_video = acer.get("https://www.acfun.cn/v/ac4741185")\nprint(demo_video)\n# 文  章: https://www.acfun.cn/a/ac16695813\ndemo_article = acer.get("https://www.acfun.cn/a/ac16695813")\nprint(demo_article)\n# 合  集: https://www.acfun.cn/a/aa6001205\ndemo_album = acer.get("https://www.acfun.cn/a/aa6001205")\nprint(demo_album)\n# 番  剧: https://www.acfun.cn/bangumi/aa5023295\ndemo_bangumi = acer.get("https://www.acfun.cn/bangumi/aa5023295")\nprint(demo_bangumi)\n# 个人页: https://www.acfun.cn/u/39088\ndemo_up = acer.get("https://www.acfun.cn/u/39088")\nprint(demo_up)\n# 动  态: https://www.acfun.cn/moment/am2797962\ndemo_moment = acer.get("https://www.acfun.cn/moment/am2797962")\nprint(demo_moment)\n# 直  播: https://live.acfun.cn/live/378269\ndemo_live = acer.get("https://live.acfun.cn/live/378269")\nprint(demo_live)\n# 分  享: https://m.acfun.cn/v/?ac=37086357\ndemo_share = acer.get("https://m.acfun.cn/v/?ac=37086357")\nprint(demo_share)\n```\n\n- - -\n\n<details>\n<summary>DEMOs</summary>\n\n**以下DEMO列举了主要的使用方法，具体请自行研究。**\n\n## 👤 主要对象\n\n+ 主对象acer示例 [acer_demo.py][acer] \n\n## 📖 综合页面对象\n\n+ 首页对象示例 [index_reader.py][index] \n+ 频道对象示例 [channel_reader.py][channel] \n+ 搜索对象示例 [search_reader.py][search] \n\n## 🔗 内容页面对象\n\n+ 番剧对象 [bangumi_demo.py][bangumi]\n+ 视频对象 [video_demo.py][video]\n+ 文章对象 [article_demo.py][article]\n+ 合集对象 [album_demo.py][album]\n+ UP主对象 [member_demo.py][member]\n+ 动态对象 [moment_demo.py][moment]\n+ 直播对象 [live_demo.py][live]\n\n## 🎁 附赠: AcSaver\n\n+ 离线保存 [AcSaver_demo.py][saver] \n\n</details>\n\n<details>\n<summary>AcSaver</summary>\n\n> 这是一个依赖acfunSDK的小工具，也算是DEMO。\n> \n> 主要用于离线收藏保存A站的各种资源。\n> 保存后，可使用浏览器打开对应页面。\n\n\n初始化本地路径\n```python\nsaver_path = r"D:\\AcSaver"\n\n# 实例化AcSaver父类\nacsaver = acer.AcSaver(saver_path)\n# 实例化后 会在路径下生成 index.html\n\n# github下载静态文件\n# https://github.com/dolaCmeo/acfunSDK/tree/assets\nacsaver.download_assets_from_github()\n\n# 下载所有Ac表情资源\nacsaver.save_emot()\n```\n\n保存文章\n```python\ndemo_article = acer.get("https://www.acfun.cn/a/ac32633020")\ndemo_article.saver(saver_path).save_all()\n```\n\n保存视频\n```python\ndemo_video = acer.get("https://www.acfun.cn/v/ac4741185")\ndemo_video.saver(saver_path).save_all()\n```\n\n~~保存番剧(暂未支持)~~\n```python\n\n```\n\n~~录制直播(暂未支持)~~\n```python\n\n```\n\n</details>\n\n<details>\n<summary>依赖库</summary>\n\n>内置+修改: 位于 `libs` 文件夹内\n>\n>+ [`you-get`](https://github.com/soimort/you-get)\n>+ [`ffmpeg_progress_yield`](https://github.com/slhck/ffmpeg-progress-yield)\n>+ [`blackboxprotobuf`](https://pypi.org/project/blackboxprotobuf/)\n\n**依赖: 包含在 `requirements.txt` 中**\n\n基础网络请求及页面解析:\n+ [`httpx`](https://pypi.org/project/httpx/)\n+ [`lxml`](https://pypi.org/project/lxml/)\n+ [`beautifulsoup4`](https://pypi.org/project/beautifulsoup4/)\n+ [`cssutils`](https://pypi.org/project/cssutils/)\n\n下载及html页面渲染:\n+ [`alive-progress`](https://pypi.org/project/alive-progress/)\n+ [`filetype`](https://pypi.org/project/filetype/)\n+ [`jinja2`](https://pypi.org/project/jinja2/)\n\nWebSocket通信及数据处理:\n+ [`websocket-client`](https://pypi.org/project/websocket-client/)\n+ [`pycryptodome`](https://pypi.org/project/pycryptodome/)\n+ [`protobuf`](https://pypi.org/project/protobuf/)\n+ [`proto-plus`](https://pypi.org/project/proto-plus/)\n+ [`rich`](https://pypi.org/project/rich/)\n+ [`psutil`](https://pypi.org/project/psutil/)\n\n</details>\n\n- - - \n## 参考 & 鸣谢\n\n+ [AcFun 助手](https://github.com/niuchaobo/acfun-helper) 是一个适用于 AcFun（ acfun.cn ） 的浏览器插件。\n+ [AcFunDanmaku](https://github.com/wpscott/AcFunDanmaku) 是用C# 和 .Net 6编写的AcFun直播弹幕工具。\n+ [实现自己的AcFun直播弹幕姬](https://www.acfun.cn/a/ac16695813) [@財布士醬](https://www.acfun.cn/u/311509)\n+ QQ频道“AcFun开源⑨课”\n+ 使用 [Poetry](https://python-poetry.org/) 构建\n\n- - - \n\n## About Me\n\n[![ac彩娘-阿部高和](https://tx-free-imgs2.acfun.cn/kimg/bs2/zt-image-host/ChQwODliOGVhYzRjMTBmOGM0ZWY1ZRCIzNcv.gif)][dolacfun]\n♂ 整点大香蕉🍌\n<img alt="AcFunCard" align="right" src="https://discovery.sunness.dev/39088">\n\n- - - \n\n[dolacfun]: https://www.acfun.cn/u/39088\n\n[acfun.cn]: https://www.acfun.cn/\n[Issue]: https://github.com/dolaCmeo/acfunSDK/issues\n[python]: https://www.python.org/downloads/\n[venv]: https://docs.python.org/zh-cn/3.8/library/venv.html\n\n[acer]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/acer_demo.py\n[index]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/index_reader.py\n[channel]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/channel_reader.py\n[search]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/seach_reader.py\n\n[bangumi]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/bangumi_demo.py\n[video]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/video_demo.py\n[article]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/article_demo.py\n[album]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/album_demo.py\n[member]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/member_demo.py\n[moment]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/moment_demo.py\n[live]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/live_demo.py\n\n[saver]: https://github.com/dolaCmeo/acfunSDK/blob/main/examples/AcSaver_demo.py\n',
    'author': 'dolacmeo',
    'author_email': 'dolacmeo@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://pypi.org/project/acfunsdk/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# coding=utf-8
import json
from bs4 import BeautifulSoup as Bs
from acfun.source import routes, apis
from acfun.page.utils import get_channel_info, get_page_pagelets, AcDanmaku

__author__ = 'dolacmeo'


class AcBangumi:
    aa_num = None
    page_obj = None
    page_pagelets = []
    bangumi_data = None
    bangumi_list = None
    vid = None
    item_id = None
    resourceType = 6
    is_404 = False

    def __init__(self, acer, aa_num: [str, int]):
        if isinstance(aa_num, str) and aa_num.startswith('aa'):
            aa_num = aa_num[2:]
        self.aa_num = str(aa_num)
        self.acer = acer
        self.loading()
        if self.is_404 is False:
            self.set_video()
            self.page_url = f"{routes['bangumi']}{self.aa_num}_36188_{self.item_id}"

    def __repr__(self):
        if self.is_404:
            return f"AcBangumi([ac{self.aa_num}]咦？世界线变动了。看看其他内容吧~)"
        title = self.bangumi_data.get('showTitle', self.bangumi_data.get('bangumiTitle', ""))
        return f"AcBangumi([ac{self.aa_num}_{self.item_id}]{title})".encode(errors='replace').decode()

    def loading(self):
        req = self.acer.client.get(routes['bangumi'] + self.aa_num)
        self.is_404 = req.status_code == 404
        if self.is_404:
            return False
        self.page_obj = Bs(req.text, 'lxml')
        newheader = self.page_obj.select_one("#pagelet_newheader")
        if newheader is None:
            print(routes['bangumi'] + self.aa_num)
        js_data = newheader.find_next_sibling("script").text.strip().split('\n')
        bangumi_data = "".join(js_data[0].split())
        bangumi_data_js_head = "window.pageInfo=window.bangumiData="
        bangumi_data_js_end = ";"
        assert bangumi_data.startswith(bangumi_data_js_head) and bangumi_data.endswith(bangumi_data_js_end)
        self.bangumi_data = json.loads(bangumi_data[len(bangumi_data_js_head):-len(bangumi_data_js_end)])
        bangumi_list = "".join(js_data[2].split())
        bangumi_list_js_head = "window.bangumiList="
        bangumi_list_js_end = ";"
        assert bangumi_list.startswith(bangumi_list_js_head) and bangumi_list.endswith(bangumi_list_js_end)
        self.bangumi_list = json.loads(bangumi_list[len(bangumi_list_js_head):-len(bangumi_list_js_end)])
        self.bangumi_data.update(get_channel_info(req.text))
        self.item_id = self.bangumi_data.get("itemId")
        self.vid = self.bangumi_data.get("videoId")
        self.page_pagelets = get_page_pagelets(self.page_obj)

    @property
    def season_data(self):
        return self.bangumi_data.get('relatedBangumis', [])

    @property
    def episode_data(self):
        return self.bangumi_list.get('items', [])

    def set_video(self, num=1):
        if num > len(self.episode_data):
            return False
        this_episode = self.episode_data[num - 1]
        self.vid = this_episode['videoId']
        self.item_id = this_episode['itemId']
        self.bangumi_data.update({
            'videoId': self.vid, 'itemId': self.item_id,
            'showTitle': f"{this_episode['bangumiTitle']} {this_episode['episodeName']} {this_episode['title']}"
        })
        self.page_url = f"{routes['bangumi']}{self.aa_num}_36188_{self.item_id}"
        return True

    def danmaku(self):
        return AcDanmaku(self.acer, self.bangumi_data)

    def comment(self):
        return self.acer.AcComment(f"{self.aa_num}_{self.vid}", 6, self.page_url)

    def like(self):
        return self.acer.like(self.item_id, 18)

    def like_cancel(self):
        return self.acer.like_cancel(self.item_id, 18)

    def favorite_add(self):
        return self.acer.favourite.add(self.aa_num, 1)

    def favorite_cancel(self):
        return self.acer.favourite.cancel(self.aa_num, 1)

    def banana(self):
        return self.acer.throw_banana(self.item_id, 18, 1)



# coding=utf-8
from rich import print
import time
import json
import gzip
import proto
import base64
import random
import filetype
import threading
from acfun.libs.blackboxprotobuf import protobuf_to_json
from hashlib import md5
from Crypto import Random
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad
from acfun.source import apis
from google.protobuf.json_format import MessageToJson
from google.protobuf.pyext.cpp_message import GeneratedProtocolMessageType
from google.protobuf.pyext._message import RepeatedScalarContainer
from .Im import PacketHeader_pb2, \
    UpstreamPayload_pb2, DownstreamPayload_pb2, \
    RegisterRequest_pb2, RegisterResponse_pb2, \
    ClientConfigGetRequest_pb2, ClientConfigGetResponse_pb2, \
    KeepAliveRequest_pb2, KeepAliveResponse_pb2, \
    GroupMemberListGetRequest_pb2, GroupMemberListGetResponse_pb2, \
    PullOldRequest_pb2, PullOldResponse_pb2, \
    SessionListResponse_pb2, SessionCreateRequest_pb2, \
    SessionCreateResponse_pb2, SessionRemoveRequest_pb2, \
    ErrorMessage_pb2, Message_pb2, Image_pb2
from .Live import ZtLiveCsCmd_pb2, \
    ZtLiveCsEnterRoom_pb2, ZtLiveCsHeartbeat_pb2, ZtLiveCsUserExit_pb2, \
    ZtLiveScMessage_pb2, acfun_live_pb2, \
    ZtLiveScActionSignal_pb2, \
    CommonActionSignalComment_pb2, CommonActionSignalLike_pb2, \
    CommonActionSignalUserEnterRoom_pb2, CommonActionSignalUserFollowAuthor_pb2, \
    CommonActionSignalGift_pb2, CommonActionSignalRichText_pb2, \
    ZtLiveScStateSignal_pb2, \
    CommonStateSignalDisplayInfo_pb2, CommonStateSignalTopUsers_pb2, \
    CommonStateSignalRecentComment_pb2, CommonStateSignalChatCall_pb2, \
    CommonStateSignalChatAccept_pb2, CommonStateSignalChatReady_pb2, \
    CommonStateSignalChatEnd_pb2, CommonStateSignalCurrentRedpackList_pb2, \
    CommonStateSignalAuthorChatCall_pb2, CommonStateSignalAuthorChatAccept_pb2, \
    CommonStateSignalAuthorChatReady_pb2, CommonStateSignalAuthorChatEnd_pb2, \
    CommonStateSignalAuthorChatChangeSoundConfig_pb2, \
    CommonStateSignalPKAccept_pb2, CommonStateSignalPKInvitation_pb2, \
    CommonStateSignalPKReady_pb2, CommonStateSignalPKSoundConfigChanged_pb2, \
    CommonStateSignalPkEnd_pb2, CommonStateSignalPkStatistic_pb2, \
    CommonStateSignalWishSheetCurrentState_pb2, \
    ZtLiveScNotifySignal_pb2, \
    CommonNotifySignalKickedOut_pb2, \
    CommonNotifySignalViolationAlert_pb2, \
    CommonNotifySignalLiveManagerState_pb2, \
    ZtLiveScStatusChanged_pb2, \
    ZtLiveScTicketInvalid_pb2


class ErrorMessage(proto.Message):
    errorCode = proto.Field(proto.INT32, number=3)
    errorMessage = proto.Field(proto.STRING, number=5)


def message_content_serialize(content: str):
    class MessageContent(proto.Message):
        content = proto.Field(proto.STRING, number=1)
    msg = MessageContent()
    msg.content = content.encode()
    return MessageContent.serialize(msg)


def proto_user_serialize(uid: int):
    class User(proto.Message):
        appId = proto.Field(proto.INT32, number=1)
        uid = proto.Field(proto.INT64, number=2)
    user = User()
    user.appId = 13
    user.uid = uid
    return User.serialize(user)


class AcProtos:
    seqId = 0
    live_room = None
    live_ticket = None
    live_heartbeat = 0
    header_offset = 12
    payload_offset = 16
    client_config = {}
    command_rules = {
        "Basic.ClientConfigGet": ClientConfigGetResponse_pb2.ClientConfigGetResponse,
        "Basic.KeepAlive": KeepAliveResponse_pb2.KeepAliveResponse,
        "Message.Session": SessionListResponse_pb2.SessionListResponse,
        "Group.UserGroupList": GroupMemberListGetResponse_pb2.GroupMemberListGetResponse,
        "Message.PullOld": PullOldResponse_pb2.PullOldResponse,
        "Session.Create": SessionCreateResponse_pb2.SessionCreateResponse,
        "Message.Send": Message_pb2.Message,
        "Push.Message": Message_pb2.Message,
        "Message.SessionRemove": None,
        "Basic.Unregister": None,
    }
    live_acks = {
        "ZtLiveCsEnterRoomAck": ZtLiveCsEnterRoom_pb2.ZtLiveCsEnterRoomAck,
        "ZtLiveCsHeartbeatAck": ZtLiveCsHeartbeat_pb2.ZtLiveCsHeartbeatAck,
        "ZtLiveCsUserExitAck": ZtLiveCsUserExit_pb2.ZtLiveCsUserExitAck,
    }
    live_messages = {
        "ZtLiveScActionSignal": ZtLiveScActionSignal_pb2.ZtLiveScActionSignal,
        "ZtLiveScActionSignal.loader": {
            "CommonActionSignalComment": CommonActionSignalComment_pb2.CommonActionSignalComment,
            "CommonActionSignalLike": CommonActionSignalLike_pb2.CommonActionSignalLike,
            "CommonActionSignalUserEnterRoom": CommonActionSignalUserEnterRoom_pb2.CommonActionSignalUserEnterRoom,
            "CommonActionSignalUserFollowAuthor": CommonActionSignalUserFollowAuthor_pb2.CommonActionSignalUserFollowAuthor,
            "AcfunActionSignalThrowBanana": acfun_live_pb2.AcfunActionSignalThrowBanana,
            "CommonActionSignalGift": CommonActionSignalGift_pb2.CommonActionSignalGift,
            "CommonActionSignalRichText": CommonActionSignalRichText_pb2.CommonActionSignalRichText,
            "AcfunActionSignalJoinClub": acfun_live_pb2.AcfunActionSignalJoinClub,
        },
        "ZtLiveScStateSignal": ZtLiveScStateSignal_pb2.ZtLiveScStateSignal,
        "ZtLiveScStateSignal.loader": {
            "AcfunStateSignalDisplayInfo": acfun_live_pb2.AcfunStateSignalDisplayInfo,
            "CommonStateSignalDisplayInfo": CommonStateSignalDisplayInfo_pb2.CommonStateSignalDisplayInfo,
            "CommonStateSignalTopUsers": CommonStateSignalTopUsers_pb2.CommonStateSignalTopUsers,
            "CommonStateSignalRecentComment": CommonStateSignalRecentComment_pb2.CommonStateSignalRecentComment,
            "CommonStateSignalChatCall": CommonStateSignalChatCall_pb2.CommonStateSignalChatCall,
            "CommonStateSignalChatAccept": CommonStateSignalChatAccept_pb2.CommonStateSignalChatAccept,
            "CommonStateSignalChatReady": CommonStateSignalChatReady_pb2.CommonStateSignalChatReady,
            "CommonStateSignalChatEnd": CommonStateSignalChatEnd_pb2.CommonStateSignalChatEnd,
            "CommonStateSignalCurrentRedpackList": CommonStateSignalCurrentRedpackList_pb2.CommonStateSignalCurrentRedpackList,
            "CommonStateSignalAuthorChatCall": CommonStateSignalAuthorChatCall_pb2.CommonStateSignalAuthorChatCall,
            "CommonStateSignalAuthorChatAccept": CommonStateSignalAuthorChatAccept_pb2.CommonStateSignalAuthorChatAccept,
            "CommonStateSignalAuthorChatReady": CommonStateSignalAuthorChatReady_pb2.CommonStateSignalAuthorChatReady,
            "CommonStateSignalAuthorChatEnd": CommonStateSignalAuthorChatEnd_pb2.CommonStateSignalAuthorChatEnd,
            "CommonStateSignalAuthorChatChangeSoundConfig": CommonStateSignalAuthorChatChangeSoundConfig_pb2.CommonStateSignalAuthorChatChangeSoundConfig,
            "CommonStateSignalPKAccept": CommonStateSignalPKAccept_pb2.CommonStateSignalPKAccept,
            "CommonStateSignalPKInvitation": CommonStateSignalPKInvitation_pb2.CommonStateSignalPKInvitation,
            "CommonStateSignalPKReady": CommonStateSignalPKReady_pb2.CommonStateSignalPKReady,
            "CommonStateSignalPKSoundConfigChanged": CommonStateSignalPKSoundConfigChanged_pb2.CommonStateSignalPKSoundConfigChanged,
            "CommonStateSignalPkEnd": CommonStateSignalPkEnd_pb2.CommonStateSignalPkEnd,
            "CommonStateSignalPkStatistic": CommonStateSignalPkStatistic_pb2.CommonStateSignalPkStatistic,
            "CommonStateSignalWishSheetCurrentState": CommonStateSignalWishSheetCurrentState_pb2.CommonStateSignalWishSheetCurrentState,
        },
        "ZtLiveScNotifySignal": ZtLiveScNotifySignal_pb2.ZtLiveScNotifySignal,
        "ZtLiveScNotifySignal.loader": {
            "CommonNotifySignalKickedOut": CommonNotifySignalKickedOut_pb2.CommonNotifySignalKickedOut,
            "CommonNotifySignalViolationAlert": CommonNotifySignalViolationAlert_pb2.CommonNotifySignalViolationAlert,
            "CommonNotifySignalLiveManagerState": CommonNotifySignalLiveManagerState_pb2.CommonNotifySignalLiveManagerState,
        },
        "ZtLiveScStatusChanged": ZtLiveScStatusChanged_pb2.ZtLiveScStatusChanged,
        "ZtLiveScTicketInvalid": ZtLiveScTicketInvalid_pb2.ZtLiveScTicketInvalid,
    }

    def __init__(self, acws):
        self.acws = acws
        self.acer = self.acws.acer

    def _aes_encrypt(self, key, payload):
        key = base64.standard_b64decode(key)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(key, AES.MODE_CBC, iv)
        payload = pad(payload, AES.block_size, 'pkcs7')
        result = cipher.encrypt(payload)
        return iv + result

    def _aes_decrypt(self, key, data):
        key = base64.standard_b64decode(key)
        iv = data[:self.payload_offset]
        payload = data[self.payload_offset:]
        cipher = AES.new(key, AES.MODE_CBC, iv)
        result = cipher.decrypt(payload)
        return unpad(result, AES.block_size, 'pkcs7')

    def decode(self, message: bytes):
        # print(f"total length: {len(message)}")
        packet_header_len = int.from_bytes(message[4:8], byteorder='big')
        payload_len = int.from_bytes(message[8:12], byteorder='big')
        # print(f"packet_header_len: {packet_header_len} {message[4:8].hex()}")
        # print(f"payload_len: {payload_len} {message[8:12].hex()}")
        packet_header = PacketHeader_pb2.PacketHeader()
        header = message[self.header_offset:self.header_offset + packet_header_len]
        packet_header.ParseFromString(header)
        # print(packet_header)
        # print("=" * 40)
        payload_data = message[self.header_offset + packet_header_len:]
        # print(f"packet_header.encryptionMode: {packet_header.encryptionMode}")
        if packet_header.encryptionMode == 1:  # kEncryptionServiceToken
            decrypted_data = self._aes_decrypt(self.acer.tokens['ssecurity'], payload_data)
        elif packet_header.encryptionMode == 2:  # kEncryptionSessionKey
            decrypted_data = self._aes_decrypt(self.acer.tokens['sessKey'], payload_data)
        else:  # kEncryptionNone
            decrypted_data = payload_data
            stream_payload = ErrorMessage.deserialize(decrypted_data)
            print(stream_payload)
            raise BufferError(self.seqId, stream_payload.errorMessage)
        # decrypted_data = decrypted_data[:packet_header.decodedPayloadLen]
        # print(f"decrypted_data[{len(decrypted_data)}]: ", decrypted_data)
        # print(f"decrypted_data[b64]: {base64.standard_b64encode(decrypted_data)}")
        if self.seqId < 2:
            stream_payload = UpstreamPayload_pb2.UpstreamPayload()
        else:
            stream_payload = DownstreamPayload_pb2.DownstreamPayload()
        stream_payload.ParseFromString(decrypted_data)
        # print("#" * 40)
        # print(f"RecvCommand: {stream_payload.command}")
        # print("#" * 40)
        if stream_payload.errorCode:
            print(stream_payload)
            raise ValueError(f"[{stream_payload.errorCode}] {stream_payload.errorMsg}")
        command = stream_payload.command
        proto_class = self.command_rules.get(command)
        if command == "Basic.Register":  # 初始化注册
            return self.BasicRegister_Response(packet_header, stream_payload)
        elif command == "Global.ZtLiveInteractive.CsCmd":  # 直播相关
            cmd_payload = ZtLiveCsCmd_pb2.ZtLiveCsCmdAck()
            cmd_payload.ParseFromString(stream_payload.payloadData)
            ack_type = cmd_payload.cmdAckType
            ack_payload = cmd_payload.payload
            if ack_type in self.live_acks:
                return self.ZtLiveCsCmdAck_Response(ack_type, packet_header, ack_payload)
            if cmd_payload.errorCode:
                print(f"LiveError[{cmd_payload.errorCode}] {cmd_payload.errorMsg}")
        elif command == "Push.ZtLiveInteractive.Message":  # 直播推送
            live_push = ZtLiveScMessage_pb2.ZtLiveScMessage()
            live_push.ParseFromString(stream_payload.payloadData)
            msg_type = live_push.messageType
            msg_payload = gzip.decompress(live_push.payload) \
                if live_push.compressionType == 2 else live_push.payload
            if msg_type in self.live_messages:
                return self.ZtLiveInteractive_Message(msg_type, packet_header, msg_payload)
            else:
                print(f"LiveMessageUnknown: {msg_type}\n{msg_payload}")
        elif isinstance(proto_class, GeneratedProtocolMessageType):  # 可解析消息
            return self.message_response(proto_class, packet_header, stream_payload)
        # 未知消息解析
        return self.unknown_response(packet_header, stream_payload)

    def BasicRegister_Response(self, packet_header, stream_payload):
        reg_resp = RegisterResponse_pb2.RegisterResponse()
        reg_resp.ParseFromString(stream_payload.payloadData)
        self.acws.appId = packet_header.appId
        self.acws.instanceId = packet_header.instanceId
        self.acer.tokens['sessKey'] = base64.standard_b64encode(reg_resp.sessKey)
        return packet_header.seqId, stream_payload.command, MessageToJson(reg_resp)

    def ZtLiveCsCmdAck_Response(self, ack_type, packet_header, ack_payload):
        if ack_type == "ZtLiveCsHeartbeatAck":
            self.acws.task(*self.ZtLiveCsHeartbeat_Request())
        payload = self.live_acks.get(ack_type)()
        payload.ParseFromString(ack_payload)
        return packet_header.seqId, f"LiveCmd.{ack_type}", json.loads(MessageToJson(payload))

    def ZtLiveInteractive_Message(self, msg_type, packet_header, msg_payload):
        self.acws.task(*self.ZtLiveInteractiveMessage_Request())
        payload = self.live_messages.get(msg_type)()
        payload.ParseFromString(msg_payload)
        those_msg = list()
        if msg_type in ["ZtLiveScActionSignal", "ZtLiveScStateSignal", "ZtLiveScNotifySignal"]:
            for item in payload.item:
                signal = item.signalType
                data = {"signal": f"{msg_type}.{signal}"}
                loader = self.live_messages.get(f"{msg_type}.loader", {}).get(f"{signal}")
                if loader is not None:
                    loader = loader()
                    if isinstance(item.payload, RepeatedScalarContainer):
                        data['payload'] = list()
                        for x in item.payload:
                            loader.ParseFromString(x)
                            data['payload'].append(json.loads(MessageToJson(loader)))
                    else:
                        loader.ParseFromString(item.payload)
                        data['payload'] = json.loads(MessageToJson(loader))
                those_msg.append(data)
        elif msg_type in ["ZtLiveScStatusChanged", "ZtLiveScTicketInvalid"]:
            those_msg = json.loads(MessageToJson(payload))
        else:
            those_msg = json.loads(MessageToJson(payload))
        if isinstance(those_msg, dict):
            those_msg = [those_msg]
        return packet_header.seqId, f"LivePush.{msg_type}", those_msg

    def message_response(self, proto_class, packet_header, stream_payload):
        payload = proto_class()
        payload.ParseFromString(stream_payload.payloadData)
        return packet_header.seqId, stream_payload.command, json.loads(MessageToJson(payload))

    def unknown_response(self, packet_header, stream_payload):
        message, typedef = protobuf_to_json(stream_payload.payloadData)
        if stream_payload.command not in self.command_rules:
            print(f"[seqId:{packet_header.seqId}] {stream_payload.command}")
            print(typedef)
            print(message)
        payload_b64 = base64.standard_b64encode(stream_payload.payloadData)
        return packet_header.seqId, stream_payload.command, payload_b64

    def encode(self, key_n: int, command: str, payload, subBiz: [str, None] = None):
        self.seqId += 1
        # ####################   UpstreamPayload    ################# #
        upstream_payload = UpstreamPayload_pb2.UpstreamPayload()
        upstream_payload.command = command
        upstream_payload.seqId = self.seqId
        upstream_payload.retryCount = 1
        if hasattr(payload, "DESCRIPTOR"):  # protos_class has attr "DESCRIPTOR"
            upstream_payload.payloadData = payload.SerializeToString()
        elif isinstance(payload, bytes):
            upstream_payload.payloadData = payload
        if isinstance(subBiz, (str, bytes)):
            upstream_payload.subBiz = subBiz
        payload_body = upstream_payload.SerializeToString()
        # ####################      Encoding        ################# #
        token_info = PacketHeader_pb2.TokenInfo__pb2.TokenInfo()
        token_info.tokenType = 1
        token_info.token = self.acer.tokens['api_st'].encode() \
            if self.acer.is_logined else self.acer.tokens['visitor_st'].encode()
        header = PacketHeader_pb2.PacketHeader()
        header.appId = self.acws.appId
        header.uid = self.acer.uid
        header.instanceId = self.acws.instanceId
        payload_len = len(payload_body)
        encrypted = payload_body
        if key_n in [1, 2]:
            key = self.acer.tokens['ssecurity'] if key_n == 1 else self.acer.tokens['sessKey']
            encrypted = self._aes_encrypt(key, payload_body)
        header.decodedPayloadLen = payload_len
        header.encryptionMode = key_n
        header.tokenInfo.CopyFrom(token_info)
        header.seqId = self.seqId
        header.kpn = b"ACFUN_APP"
        header_payload = header.SerializeToString()
        return self.seqId, command, b"".join([
            bytes.fromhex("abcd0001"),
            len(header_payload).to_bytes(4, "big"),
            len(encrypted).to_bytes(4, "big"),
            header_payload,
            encrypted
        ])

    def BasicRegister_Request(self):
        payload = RegisterRequest_pb2.RegisterRequest()
        app_info = RegisterRequest_pb2.AppInfo__pb2.AppInfo()
        app_info.sdkVersion = "kwai-acfun-live-link"
        app_info.linkVersion = "2.13.8"
        payload.appInfo.CopyFrom(app_info)
        device_info = RegisterRequest_pb2.DeviceInfo__pb2.DeviceInfo()
        device_info.platformType = 9
        device_info.deviceModel = "h5"
        if self.acer.is_logined:
            device_info.deviceId = self.acer.did
        payload.deviceInfo.CopyFrom(device_info)
        payload.presenceStatus = 1
        payload.appActiveStatus = 1
        ztcommon_info = RegisterRequest_pb2.ZtCommonInfo__pb2.ZtCommonInfo()
        ztcommon_info.kpn = "ACFUN_APP"
        ztcommon_info.kpf = "PC_WEB"
        ztcommon_info.uid = int(self.acer.uid)
        if self.acer.is_logined:
            ztcommon_info.did = self.acer.did
        payload.instanceId = self.acws.instanceId
        payload.ztCommonInfo.CopyFrom(ztcommon_info)
        return self.encode(1, "Basic.Register", payload, "mainApp")

    def ClientConfigGet_Request(self):
        payload = ClientConfigGetRequest_pb2.ClientConfigGetRequest()
        payload.version = 0
        return self.encode(2, "Basic.ClientConfigGet", payload)

    def KeepAlive_Request(self):
        payload = KeepAliveRequest_pb2.KeepAliveRequest()
        payload.presenceStatus = 1
        payload.appActiveStatus = 1
        payload.keepaliveIntervalSec = 120
        return self.encode(2, "Basic.KeepAlive", payload)

    def Unregister_Request(self):
        return self.encode(2, "Basic.Unregister", bytes())

    def BasicPing_Request(self):
        return self.encode(2, "Basic.Ping", bytes())

    def MessageSession_Request(self):
        return self.encode(2, "Message.Session", bytes([16, 0]))

    def SessionCreate_Request(self, target_uid: int):
        payload = SessionCreateRequest_pb2.SessionCreateRequest()
        chat = SessionCreateRequest_pb2.ChatTarget__pb2.ChatTarget()
        chat.targetId = f"{target_uid}"
        chat.targetType = 0
        payload.chatTarget.CopyFrom(chat)
        return self.encode(2, "Session.Create", payload)

    def SessionRemove_Request(self, target_uid: int):
        payload = SessionRemoveRequest_pb2.SessionRemoveRequest()
        payload.targetId = target_uid
        payload.strTargetId = f"{target_uid}"
        return self.encode(2, "Message.SessionRemove", payload)

    def UserGroupList_Request(self):
        payload = GroupMemberListGetRequest_pb2.GroupMemberListGetRequest()
        payload.groupId = b"0"
        sync_cookie_payload = GroupMemberListGetRequest_pb2.SyncCookie__pb2.SyncCookie()
        sync_cookie_payload.syncOffset = -1
        payload.syncCookie.CopyFrom(sync_cookie_payload)
        return self.encode(2, "Group.UserGroupList", payload)

    def MessagePullOld_Request(self, uid: int, minSeq: int, maxSeq: int, count: int = 10):
        payload = PullOldRequest_pb2.PullOldRequest()
        payload.target.ParseFromString(proto_user_serialize(uid))
        payload.minSeq = minSeq
        payload.maxSeq = maxSeq
        payload.count = count
        return self.encode(2, "Message.PullOld", payload)

    def Message_Request(self, target_uid: int, content: bytes, content_type: int = 0):
        payload = Message_pb2.Message()
        # payload.seqId = int(f"{time.time():.0f}{1:0>6}")
        payload.clientSeqId = int(f"{time.time():.0f}{1:0>6}")
        payload.timestampMs = int(f"{time.time():.0f}{1:0>3}")
        payload.fromUser.ParseFromString(proto_user_serialize(self.acer.uid))
        payload.targetId = target_uid
        payload.toUser.ParseFromString(proto_user_serialize(target_uid))
        payload.contentType = content_type
        payload.content = content
        payload.strTargetId = str(target_uid)
        return self.encode(2, "Message.Send", payload)

    def MessageContent_Request(self, target_uid: int, content: str):
        byte_content = message_content_serialize(content)
        return self.Message_Request(target_uid, byte_content, 0)

    def im_image_uploader(self, to_uid: str, image_data: bytes):
        md5_hex = md5(image_data).digest()
        head = {
            "Content-Type": filetype.guess_mime(image_data),
            "Content-MD5": base64.standard_b64encode(md5_hex),
            "file-type": filetype.guess_extension(image_data),
            "target": f"{to_uid}",
            "download-verify-type": "1",
        }
        param = {'kpn': "ACFUN_APP"}
        param = self.acer.update_token(param)
        post_req = self.acer.client.post(apis['im_image_upload'], params=param, headers=head, data=image_data)
        return post_req.json()

    def MessageImage_Request(self, target_uid: int, image_data: bytes):
        uploader_resp = self.im_image_uploader(to_uid=f"{target_uid}", image_data=image_data)
        ks_uri = uploader_resp.get('uri')
        if ks_uri is None:
            print(uploader_resp)
            raise ValueError(uploader_resp['error_msg'])
        img_payload = Image_pb2.Image()
        img_payload.uri = ks_uri
        img_payload.width = 300
        img_payload.height = 300
        img_payload.contentLength = len(image_data)
        byte_content = img_payload.SerializeToString()
        return self.Message_Request(target_uid, byte_content, 1)

    def ZtLiveInteractive_CsCmd(self, cmd: str, cmd_payload):
        if self.live_room is None:
            raise ValueError(f"Need Enter Live Rome First!!!\nUSE: ZtLiveCsEnterRoom_Request(uid)")
        payload = ZtLiveCsCmd_pb2.ZtLiveCsCmd()
        payload.cmdType = cmd
        payload.payload = cmd_payload.SerializeToString()
        payload.liveId = self.live_room.get("liveId")
        payload.ticket = self.live_ticket
        return self.encode(2, "Global.ZtLiveInteractive.CsCmd", payload, "mainApp")

    def get_aclive_room_info(self, uid: int):
        param = {
            "subBiz": "mainApp",
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "userId": self.acer.uid,
            "did": self.acer.did
        }
        param = self.acer.update_token(param)
        form_data = {"authorId": uid, "pullStreamType": "FLV"}
        api_req = self.acer.client.post(apis['live_play'], params=param, data=form_data,
                                        headers={'referer': "https://live.acfun.cn/"})
        api_data = api_req.json()
        if api_data.get('result') != 1:
            self.acws.close()
            raise ValueError(f"{api_data['result']}: {api_data['error_msg']}")
        self.live_room = api_data.get('data')
        self.live_ticket = random.choice(self.live_room.get("availableTickets"))
        return self.live_room is not None

    def get_live_gift_data(self):
        if self.live_room is None:
            return None
        param = {
            "subBiz": "mainApp",
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "userId": self.acer.uid,
            "did": self.acer.did
        }
        param = self.acer.update_token(param)
        form_data = {
            "visitorId": self.acer.uid,
            "liveId": self.live_room.get("liveId"),
        }
        api_req = self.acer.client.post(apis['live_gift_list'], params=param, data=form_data,
                                        headers={'referer': "https://live.acfun.cn/"})
        api_data = api_req.json()
        gifts = dict()
        for gift in api_data.get('data', {}).get('giftList', []):
            gifts.update({f"{gift['giftId']}": gift})
        ext_gift = api_data.get('data', {}).get('externalDisplayGift', {}).get('giftList', [])
        if len(ext_gift):
            ext_gift = ext_gift[0]
            ext_gift.update({"tipsDelayTime": api_data.get('data', {}).get('externalDisplayGiftTipsDelayTime', 0)})
            gifts.update({f"{ext_gift['giftId']}": ext_gift})
        return gifts

    def ZtLiveCsEnterRoom_Request(self, uid: int):
        self.get_aclive_room_info(uid)
        enter_room_payload = ZtLiveCsEnterRoom_pb2.ZtLiveCsEnterRoom()
        enter_room_payload.isAuthor = self.acer.uid == uid
        # enter_room_payload.reconnectCount = 0
        # enter_room_payload.lastErrorCode = 0
        enter_room_payload.enterRoomAttach = self.live_room.get("enterRoomAttach")
        enter_room_payload.clientLiveSdkVersion = "kwai-acfun-live-link"
        return self.ZtLiveInteractive_CsCmd("ZtLiveCsEnterRoom", enter_room_payload)

    def ZtLiveCsHeartbeat_Request(self):
        self.acws.task(*self.ZtLiveInteractiveMessage_Request())
        self.live_heartbeat += 1
        if self.live_heartbeat % 5 == 4:
            # 即启动Heartbeat定时器后每50秒，发送KeepAliveRequest，SeqId + 1
            self.acws.task(*self.KeepAlive_Request())
        time.sleep(4)
        heartbeat_payload = ZtLiveCsHeartbeat_pb2.ZtLiveCsHeartbeat()
        heartbeat_payload.clientTimestampMs = int(time.time()) * 1000
        heartbeat_payload.sequence = self.live_heartbeat
        return self.ZtLiveInteractive_CsCmd("ZtLiveCsHeartbeat", heartbeat_payload)

    def ZtLiveInteractiveMessage_Request(self):
        if self.live_room is None:
            raise ValueError(f"Need Enter Live Rome First!!!\nUSE: ZtLiveCsEnterRoom_Request(uid)")
        return self.encode(2, "Push.ZtLiveInteractive.Message", bytes(), "mainApp")




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: GroupJoinStatus.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15GroupJoinStatus.proto\x12\x1b\x41\x63\x46unDanmu.Im.Cloud.Message*[\n\x0fGroupJoinStatus\x12\x11\n\rINVITE_STATUS\x10\x00\x12\t\n\x05\x41GREE\x10\x01\x12\n\n\x06REFUSE\x10\x02\x12\x0c\n\x08\x41PPLYING\x10\x03\x12\x10\n\x0cOUT_OF_GROUP\x10\x04\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'GroupJoinStatus_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _GROUPJOINSTATUS._serialized_start=54
  _GROUPJOINSTATUS._serialized_end=145
# @@protoc_insertion_point(module_scope)
